/**
 * Qt C++ Wrapper for kdtree
 *
 * Stores QVariants with specific positions in a k-d tree.
 *
 * Ray Rischpater, Nokia Corporation
 *
 * Ray.Rischpater@Nokia.com
 *
 * kdtree module Copyright (C) 2007-2011 John Tsiombikas <nuclear@member.fsf.org>
 * See kdtree.cpp for licensing information.
 */

#ifndef KDTREE_H
#define KDTREE_H
#include <QVariant>
#include <QPointF>
#include <QVector>
#include <QPair>

struct kdtree;

class VariantKDNode {
public:
    VariantKDNode(const QVector<double>& position, const QVariant& data );
    VariantKDNode(const double* position, const unsigned int dimension, const QVariant &data);
    QVector<double> position() const;
    QVariant data() const;

private:
    QVector<double> mPosition;
    QVariant mData;
};

class VariantKDTree
{
public:
    /** Creates a k-dimensional tree. */
    VariantKDTree(const int k = 2);

    /** Destructor. */
    ~VariantKDTree();

    /** Inserts data at position */
    bool add(const double* position, const QVariant& data);
    bool add(const QPointF& position, const QVariant& data);
    bool add(const QVector<double>& position, const QVariant& data);
    bool add(const VariantKDNode& node);

    /** Finds nearest item to position */
    VariantKDNode nearest(const double* position);
    VariantKDNode nearest(const QVector<double>& position);
    VariantKDNode nearest(const QPointF& position);

    /** Finds n items in range of position */
    QList<VariantKDNode> find(const double* position, const double range);
    QList<VariantKDNode> find(const QPointF& position, const double range);
    QList<VariantKDNode> find(const QVector<double>& position, const double range);

private:
    struct kdtree* mTree;
    QList<QVariant*> mData;
};

#endif // KDTREE_H
