#include <QtCore/QCoreApplication>
#include <QDebug>

#include "kdtree.h"
#include "quadtree.h"

const double  xs[] = {  6, 11,  4,  1,  1,  2,  2,  1.5, 13, 14, 14, 14, 15 };
const double  ys[] = {  3,  5, 16, 18, 19, 18, 19, 18.5, 16, 16, 17, 18, 17 };

const double rxs[] = {  0.5,    7.0,   5.0, 12.5,  16.0 };
const double rys[] = { 17.0,    7.0,   2.0, 15.0,   2.0 };
const double rs[] = {  2.5, 2, 2, 2, 2};
const double wis[] = {  2.5, 2, 2, 2, 2};
const double hes[] = {  2.5, 2, 2, 1, 2};


int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);

    qDebug() << "Hello world.";


    qDebug() << "KD-TREE";

    // Create kd-tree.
    VariantKDTree tree;

    // Add items to kd-tree.
    for(unsigned int i = 0; i < sizeof(xs)/sizeof(xs[0]); i++) {
        QPointF p(xs[i], ys[i]);
        QString data(QString("%1, %2").arg(xs[i]).arg(ys[i]));
        tree.add(p, data);
        qDebug() << i << xs[i] << "," << ys[i];
    }

    // Search for items in regions of kd-tree.
    for(unsigned int i = 0; i < sizeof(rxs)/sizeof(rxs[0]); i++) {
        QPointF p(rxs[i], rys[i]);
        qDebug() << p << rs[i] << "x" << rs[i];
        QList<VariantKDNode> nodes = tree.find(p, rs[i]);
        foreach(const VariantKDNode& aNode, nodes) {
            qDebug() << aNode.position() << aNode.data().toString();
        }
    }

    qDebug() << "QUAD TREE";

    QuadTree<QString> qt(QPointF(0,0), QSizeF(20, 20));

    for(unsigned int i = 0; i < sizeof(xs)/sizeof(xs[0]); i++) {
        QString data(QString("%1, %2").arg(xs[i]).arg(ys[i]));
        QuadTreeNode<QString> node(QPointF(xs[i], ys[i]), data);
        qt.add(node);
        qDebug() << i << xs[i] << "," << ys[i];
    }

    for(unsigned int i = 0; i < sizeof(rxs)/sizeof(rxs[0]); i++) {
        QRectF region(QPointF(rxs[i], rys[i]), QSizeF(wis[i], hes[i]));
        QList< QuadTreeNode<QString> > nodes = qt.find(region);
        qDebug() << region;
        foreach(const QuadTreeNode<QString>& aNode, nodes) {
            qDebug() << " " << aNode.data();
        }
    }


    // Call it quits
    qDebug() << "Goodbye world.";

    return a.exec();
}
